#!/usr/bin/env python
"""File for automising the translation process.
Run this when you need updated .po and .pot files.
Run this when you need updated .mo files.
Add languages by modifying the languages variable.
Automatically adds new files to bzr.
NOTE: you will need to install intltool for this to work.
"""

import os, sys
from subprocess import call

# All supported languages should be added here
languages = ("en_GB", )

def main():
    
    
    dp = determine_path()
    pp = os.path.join(dp[:-1*len(dp.split("/")[-1])], "epidermis")
    # pp should be the epidermis dir with all the .py files in it
    
    for file in os.listdir(os.path.join(pp, "data")):
        if file[-6:] == ".glade":
            call("intltool-extract --type=gettext/glade data/%s" % file, shell=True, cwd=pp)
    call("bzr add data/*.glade.h", shell=True, cwd=pp)
    
    call("mkdir -p translation", shell=True, cwd=pp)
    
    call("xgettext --language=Python --keyword=_ --keyword=N_ --output=translation/epidermis.pot *.py pigments/*.py data/*.glade.h", shell=True, cwd=pp)
    call("bzr add translation/epidermis.pot", shell=True, cwd=pp)
    
    for lang in languages:
        if not os.path.exists(os.path.join(pp, "translation", lang + ".po")):
            # note: cwd is pp/translation here
            call("msginit --input=epidermis.pot --locale=%s --no-translator" % lang, shell=True, cwd=os.path.join(pp, "translation"))
            call("bzr add translation/%s.po" % lang, shell=True, cwd=pp)
        else:
            # note: cwd is pp/translation here
            call("msgmerge -U %s.po epidermis.pot" % lang, shell=True, cwd=os.path.join(pp, "translation"))
        call("mkdir -p translation/%s/LC_MESSAGES" % lang, shell=True, cwd=pp)
        call("msgfmt --output-file=translation/%s/LC_MESSAGES/epidermis.mo translation/%s.po" % (lang, lang), shell=True, cwd=pp)
        call("bzr add translation/%s/LC_MESSAGES/epidermis.mo" % lang, shell=True, cwd=pp)
        
    
def determine_path ():
    """Borrowed from wxglade.py"""
    try:
        root = __file__
        if os.path.islink (root):
            root = os.path.realpath (root)
        return os.path.dirname (os.path.abspath (root))
    except Exception, ee:
        print >> sys.stderr, "I'm sorry, but something is wrong."
        print >> sys.stderr, "There is no __file__ variable. Please contact the author."
        sys.exit (1)
        raise(ee)

if __name__ == "__main__":
    main()