#!/usr/bin/env python
# David D Lowe (c) 2008
# licensed under GPLv2
"""loadrepo.py provides loading and displaying of the repository's information
in a EpidermisApp's window
I aim to rewrite quite a lot of this for a repo in GHNS (Get Hot New Stuff) format"""

from __future__ import division
from const import *
import pygtk
pygtk.require("2.0")
import gtk, gobject,os,sys,threading,httplib,urlparse,socket,urllib,gconf,subprocess, gconf
import epidermis
import managepigments
from glob import glob
import handy
import ep_exceptions
from gettext import gettext as _

_acceptedEpidermisVersions = ("0.1", )

def load_repo(application, forceReload=False):
    """Loads epidermis repository, downloads and displays it
    application: the EpidermisApp object
    forceReload: whether to ignore application.repoLoaded or not, if forceReload 
        is False, load_repo will will only activate if app.repoLoaded is False"""
    app = application
    if forceReload:
        app.repoLoaded = False

    if not app.repoLoaded:
        wTree = gtk.glade.XML(app.wTreeFileName)
        fetchWin = wTree.get_widget("winOperations")
        fetchWin.set_destroy_with_parent(True)
        progressbar = wTree.get_widget("progressbarOperations")
        cancelButton = wTree.get_widget("butOperationsCancel")
        progressbar.set_fraction(0)
        
        def fetchWin_delete_callback(widget, event):
            cancel_callback(cancelButton)
            return True
        handleridDelete = fetchWin.connect("delete-event", fetchWin_delete_callback)
        
        app.tvPigments = {}
        for pt in PIGMENT_TYPES:
            app.tvPigments[pt] = app.wTree.get_widget("treeviewRepo" + managepigments.get_pigment_type(pt).directoryName.capitalize())
            if app.tvPigments[pt] is None:
                print >> sys.stderr, "Cannot find app.tvPigments[" + pt + "]"
            
        app.cancel = False
        def cancel_callback(widget, data=None):
            """cancel_callback for the cancel button on the download window"""
            app.cancel = True
            gobject.idle_add(lambda:fetchWin.hide())
            gobject.idle_add(lambda:app.switch_to_toggled(app.tgInstalled))
            gobject.idle_add(lambda:app.tgInstalled.set_active(True))
            gobject.idle_add(lambda:fetchWin.disconnect(handleridDelete))
            gobject.idle_add(lambda:cancelButton.disconnect(handleridCancel))
        handleridCancel = cancelButton.connect("clicked", cancel_callback)
        
        class CancelException(Exception):
            """exception that may be raised by the download_hook function"""
            def __str__(self): return "CancelException"
        
        def download_hook(blocksDownloaded, blockSize, size):
            """download_hook for the download thread that displays progress
            in the progressbar"""
            downloaded = blocksDownloaded * blockSize
            frac = blocksDownloaded * blockSize/ size
            if frac > 1: frac = 1
            if frac < 0: frac = 0
            gtk.gdk.threads_enter()
            progressbar.set_fraction(frac)
            if size >= 0:
                progressbar.set_text(handy.bytes_in_str(downloaded) + "/" + handy.bytes_in_str(size))
            else:
                progressbar.set_text(handy.bytes_in_str(downloaded) + "/??")
            gtk.gdk.threads_leave()
            if app.cancel:
                raise(CancelException())
        
        def load_cached_repo():
            """loads repo from cache"""
            
            def get_picture(ff):
                """Returns picture object for preview"""
                try:
                    return gtk.gdk.pixbuf_new_from_file_at_size(os.path.join( \
                        MY_CACHE_HOME, ff), \
                        100,75)
                except Exception, ee:
                    if os.path.exists(os.path.join(MY_CACHE_HOME,ff)):
                        print "gtk.gdk.pixbuf_new_from_file_at_size crashed loading %s" % \
                            os.path.join(MY_CACHE_HOME,ff)
                        print ee
                    else:
                        print >> sys.stderr,  "file not found for %s" % os.path.join(MY_CACHE_HOME,ff)
                    try:
                        return app.win.render_icon(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_BUTTON)
                    except Exception, ee:
                        if DEBUG:
                            raise(ee)
                        else:
                            return None
            
            # still under load_cached_repo
            # read the downloaded preview_summary.xml
            filename = os.path.join(MY_CACHE_HOME, "preview_summary.xml")
            if not os.path.exists(filename):
                app.show_error(_("No cached repo information, try again"), terminalText=_("No cached preview_summary.xml"),parent=fetchWin)
                clear_repo_cache(app)
            else:
                # preview_summary.xml exists
                # try to read and display the information
                try:
                    from xml.dom import minidom
                    thexml = minidom.parse(filename)
                except xml.parsers.expat.ExpatError, e:
                    app.show_error(_("Invalid repo information"), terminalText=_("Invalid xml: %s") % filename, parent=fetchWin)
                    clear_repo_cache(app)
                else:
                    # valid XML in preview_summary.xml
                    app.pigmentsData = {}
                    app.pigmentsToInstall = {}
                    for pt in PIGMENT_TYPES[1:] + ["skin"]:
                        app.pigmentsToInstall[pt] = {}
                        app.pigmentsData[pt] = app.PigmentTypeData(pt, app.installed_toggled, app.tvPigments[pt])
                        items = thexml.getElementsByTagName(managepigments.get_pigment_type(pt).directoryName)[0]
                        for item in items.getElementsByTagName("item"):
                            try:
                                newPigmentInstalled = False
                                newPigmentCodename = ""
                                newPigmentCodename = handy.get_first_text(item.getElementsByTagName("codename")[0])
                                newPigmentInstalled = app.is_pigment_installed(pt, newPigmentCodename)
                            except IndexError, ee:
                                app.show_error(_("Missing element in item %(id)s in %(filename)s") % {"id":itemId,
                                    "filename":filename})
                            except ep_exceptions.NoFirstTextError, ee:
                                app.show_error(_("Missing text in xml of item %(id)s in %(filename)s") % {"id":itemId,
                                    "filename":filename})
                            newPigment = None
                            if newPigmentInstalled:
                                for myPigm in app.installedPigments[pt]:
                                    if myPigm.codeName == newPigmentCodename:
                                        newPigment = myPigm
                                        break
                                if newPigment is None:
                                    raise(Exception("Cannot find installed pigment"))
                                app.pigmentsData[pt].append_to_listStore(newPigment, installed=True)
                            else: # pigment is not installed
                                newPigment = managepigments.create_pigment(pt)
                                itemId = item.getAttribute("id")
                                try:
                                    newPigment.set_human_name(handy.get_first_text(item.getElementsByTagName("name")[0]))
                                    newPigment.codeName = newPigmentCodename
                                    newPigment.set_description(handy.get_first_text(item.getElementsByTagName("description")[0]))
                                    newPigment.author = handy.get_first_text(item.getElementsByTagName("author")[0])
                                except IndexError, ee:
                                    app.show_error(_("Missing element in item %(id)s in %(filename)s") % {"id":itemId,
                                        "filename":filename})
                                except ep_exceptions.NoFirstTextError, ee:
                                    app.show_error(_("Missing text in xml of item %(id)s in %(filename)s") % {"id":itemId,
                                        "filename":filename})
                                if len(item.getElementsByTagName("preview")) > 0:
                                    try:
                                        newPigment.set_preview_file(os.path.join(MY_CACHE_HOME, handy.get_first_text(item.getElementsByTagName("preview")[0])))
                                    except ep_exceptions.NoFirstTextError:
                                        print >> sys.stderr, "no first text for preview in %s" % filename
                                #newPigmentInstalled = app.is_pigment_installed(pt,newPigment.codeName)
                                ##app.pigmentsData[pt].append_pigment_to_listStore(newPigment)
                                if pt != "skin":
                                    app.pigmentsData[pt].append_to_listStore(newPigment, newPigmentInstalled)
                                else:
                                    for ptd in PIGMENT_TYPES[1:]:
                                        if len(item.getElementsByTagName(ptd + "Dependancy")) > 0:
                                            try:
                                                linkedCodename = handy.get_first_text(item.getElementsByTagName(ptd + "Dependancy")[0])
                                            except ep_exceptions.NoFirstTextError:
                                                print >> sys.stderr, "no first text for %sDependancy in %s in %s" % (ptd, newPigment.codeName, filename)
                                            else:
                                                foundPigment = None
                                                for pt2 in PIGMENT_TYPES:
                                                    for pigm in app.pigmentsData[pt2].pigments:
                                                        if pigm.type.codeName == ptd and pigm.codeName == linkedCodename:
                                                            foundPigment = pigm
                                                            break
                                                if foundPigment is None:
                                                    print "looking for pigment " + linkedCodename + " of type " + ptd
                                                    raise(Exception("can't find pigment"))
                                                newPigment.set_linked_pigment(foundPigment)
                                        else:
                                            print >> sys.stderr, "No dep %s for %s in %s" % (ptd, newPigment.codeName, filename)
                                            raise(Exception("missing dep"))
                                    app.pigmentsData[pt].append_to_listStore(newPigment, newPigmentInstalled)

                    app.originalPigmentsListStore = {}
                    for pp in app.pigmentsData:
                        app.originalPigmentsListStore[app.pigmentsData[pp].get_pigmentType()] = app.pigmentsData[pp].blank_listStore()
                        ls = app.originalPigmentsListStore[app.pigmentsData[pp].get_pigmentType()]
                        coun = 0
                        while (coun < app.pigmentsData[pp].get_listStore().iter_n_children(None)):
                            app.pigmentsData[pp].append_to_listStore(app.pigmentsData[pp].get_pigment(coun), listStore=ls, \
                                installed=app.pigmentsData[pp].get_installed(coun))    
                            coun += 1
                    app.repoLoaded = True
                    gobject.idle_add(lambda:app.tgFindMore.set_active(True))
                    
        def prepare_downloaded_repo(filename):
            """this function extracts the downloaded repo archive in the
            appropriate places in the cache and then runs load_cached_repo()"""
            sFilename = filename.split("/")[-1:][0]
            subprocess.call(["rm", os.path.join(MY_CACHE_HOME, "preview_summary.xml")])
            subprocess.call(["mkdir", "-p", os.path.join(MY_CACHE_HOME)])
            subprocess.call(["mv", filename, os.path.join(MY_CACHE_HOME, sFilename)])
            subprocess.call(["gunzip", sFilename], cwd=MY_CACHE_HOME)
            subprocess.call(["tar", "-xf", sFilename[0:-3]], cwd=MY_CACHE_HOME, stdout=os.tmpfile())
            subprocess.call(["rm", os.path.join(MY_CACHE_HOME, sFilename[0:-3])])
            if not os.path.exists(os.path.join(MY_CACHE_HOME, "preview_summary.xml")):
                app.show_error(_("Invalid archive downloaded, does not contain preview_summary.xml"))
                clear_repo_cache(app)
            else:
                # preview_summary.xml exists
                try:
                    from xml.dom import minidom
                    thexml = minidom.parse(os.path.join(MY_CACHE_HOME, "preview_summary.xml"))
                except xml.parsers.expat.ExpatError, e:
                    app.show_error(terminalText="Invalid xml: " + os.path.join(MY_CACHE_HOME, "preview_summary.xml"), \
                        text="Invalid repo information", parent=fetchWin)
                    clear_repo_cache(app)
                else:
                    load_cached_repo()

            
                
        class DownloadRepoThread(threading.Thread):
            """thread that downloads the file using urllib.urlretrieve"""
            
            def __init__(self, url, hook, app, fetchWin):
                """url: url of repo file to download
                hook: hook function for urllib.urlretrieve
                app: EpidermisApp"""
                self.url = url
                self.hook = hook
                self.parent = app
                self.fetchWin = fetchWin
                #self.previewSummaryTgz = previewSummaryTgz
                threading.Thread.__init__(self)
                self.setName("DownloadRepoThread")
                
            def run(self):
                if DEBUG:
                    self.run2()
                else:
                    try:
                        self.run2()
                    except Exception, ee:
                        # TODO: better error handling
                        #import gtkexcepthook
                        #print gtkexcepthook.analyse(ee.__class__, ee, None)
                        self.display_download_critical_error(str(ee), _("Error loading repo"), \
                            _("An unexpected error occured: %(errortext)s") % {"errortext":str(ee)})
                        raise
                
            def run2(self):
                gtk.gdk.threads_enter()
                progressbar.set_text(_("Checking connection"))
                gtk.gdk.threads_leave()
                

                from xml.dom import minidom
                conn = httplib.HTTPConnection(urlparse.urlparse(self.parent.repoURL)[1])
                conn.request("GET", urlparse.urlparse(self.parent.repoURL)[2])
                try:
                    r = conn.getresponse()
                except IOError, e:
                    self.display_download_critical_error(_("Connection cannot be made to %s") % self.repoURL, \
                        friendlySecondaryText=_("Please verify your Internet connection"))
                    conn.close()
                    return
                except socket.timeout, e:
                    self.display_download_critical_error(_("Timeout error"))
                    conn.close()
                    return
                else:
                    if r.status == 200 or r.status == 301 or r.status == 302:
                        # no error
                        conn.close()
                    elif r.status == 404:
                        self.display_download_critical_error(_("Cannot find the file %s") % self.parent.repoURL,
                            friendlySecondaryText=_("Please make sure that the repo's URL is correct"))
                        conn.close()
                        return
                    else:
                        self.display_download_critical_error(_("Error number %s received from server") % r.status, \
                        friendlySecondaryText=_("Please make sure that the repo's URL is correct"))
                        conn.close()
                        return
                gtk.gdk.threads_enter()
                progressbar.set_text(_("Checking if repo information has changed"))
                gtk.gdk.threads_leave()
                try:
                    oo = urllib.urlopen(self.parent.repoURL)
                except (IOError), e:
                    self.display_download_critical_error(_("Connection cannot be made to %s") % self.repoURL, \
                        friendlySecondaryText=_("Please verify your Internet connection"))
                    return
                thexml = minidom.parse(oo)
                root = thexml.getElementsByTagName("ghnsdownload")[0]
                validVer = False
                for ver in root.getAttribute("epidermisVersion").split(";"):
                    if ver in _acceptedEpidermisVersions:
                        validVer = True
                        break
                if not validVer:
                    self.display_download_critical_error(_("The repo's version (%s) is not supported by this version of Epidermis. Epidermis may be out of date") % root.getAttribute("epidermisVersion"))
                    print >> sys.stderr, str(ep_exceptions.PigmentVersionNotAccepted(root.getAttribute("epidermisVersion"), _acceptedEpidermisVersions))
                    return
                previewSummaryTgzXml = None
                for item in thexml.getElementsByTagName("stuff"):
                    if item.getAttribute("category") == "epidermis/repo_preview_summary":
                        previewSummaryTgzXml = handy.get_first_text(item.getElementsByTagName("payload")[0])
                if previewSummaryTgzXml is None:
                    print "self.parent.repoURL", self.parent.repoURL
                    raise(Exception("cannot find stuff element in XML for repo_preview_summary"))
                useCached = False
                server = urlparse.urlparse(self.parent.repoURL) # eg: server[1] = localhost
                if urlparse.urlparse(previewSummaryTgzXml)[1] == "":
                    serverPath = urlparse.urlparse(self.parent.repoURL)[2][: \
                        (-1*len(self.parent.repoURL.split("/")[-1]))] # eg: serverPath = /ghns
                    previewSummaryTgz = urlparse.urljoin(server[0] + "://", server[1])
                    previewSummaryTgz = urlparse.urljoin(previewSummaryTgz.replace("///", "//"), serverPath)
                    previewSummaryTgz = urlparse.urljoin(previewSummaryTgz, previewSummaryTgzXml)
                else: # absolute URL
                    previewSummaryTgz = previewSummaryTgzXml
                conn = httplib.HTTPConnection(server[1])
                headers = {}
                if self.parent.repoLastModified != "":
                    headers["If-Modified-Since"] = self.parent.repoLastModified
                if self.parent.repoETag != "":
                    headers["If-None-Match"] = self.parent.repoETag
                try:
                #if True:
                    if self.parent.repoLastModified != "" or self.parent.repoETag != "":
                        conn.request("GET", urlparse.urlparse(previewSummaryTgz)[2], headers=headers)
                    else:
                        conn.request("GET", urlparse.urlparse(previewSummaryTgz)[2])
                except Exception, ee:
                    self.display_download_critical_error(_("Initial connection error, cannot connect to %s") % previewSummaryTgz)
                    print >> sys.stderr, str(ee)
                    if DEBUG:
                        print "server", server, "serverPath", serverPath, "previewSummaryTgz", previewSummaryTgz
                        raise(ee)
                    else:
                        return
                try:
                    r = conn.getresponse()
                except Exception, err:
                    self.display_download_critical_error(_("Connection error, cannot connect to %s") % previewSummaryTgz)
                    conn.close()
                    print >> sys.stderr, str(ee)
                    return
                firstSize = int(r.getheader("Content-Length", "-1"))
                conn.close()
                if r.status == 304:
                    # use cached copy
                    useCached = True
                    #gtk.gdk.threads_enter()
                    load_cached_repo()
                    gobject.idle_add(lambda:self.fetchWin.hide())
                    #gtk.gdk.threads_leave()
                elif r.status != 200 and r.status != 301 and r.status != 302:
                    # error, don't downoload
                    self.display_download_critical_error(_("Could not connect to %(previewSummaryTgz)s, %(status)s, %(reason)s") % \
                        {"previewSummaryTgz":previewSummaryTgz, "status":r.status, "reason":r.reason})
                else:
                    gtk.gdk.threads_enter()
                    progressbar.set_text(_("Downloading"))
                    gtk.gdk.threads_leave()
                    try:
                        filename, headers = urllib.urlretrieve(previewSummaryTgz, reporthook=self.hook)
                    except CancelException, e:
                        gobject.idle_add(lambda:self.parent.switch_to_toggled(self.parent.tgInstalled))
                    except IOError, err:
                        self.display_download_critical_error(_("Error connecting to %s\n %s") % (previewSummaryTgz,err))
                    except socket.timeout, err:
                        self.display_download_critical_error(_("Error connecting to %s, timed out\n %s") % (previewSummaryTgz, err))
                    else:
                        client = gconf.client_get_default()
                        if headers.has_key("Last-Modified"):
                            client.set_string("/apps/epidermis/repo_last_modified", headers["Last-Modified"])
                        else:
                            client.set_string("/apps/epidermis/repo_last_modified", "")
                        if headers.has_key("ETag"):
                            client.set_string("/apps/epidermis/repo_etag", headers["ETag"])
                        else:
                            client.set_string("/apps/epidermis/repo_etag", "")
                        gtk.gdk.threads_enter()
                        prepare_downloaded_repo(filename)
                        gobject.idle_add(lambda:self.fetchWin.hide())
                        gtk.gdk.threads_leave()
            
            # still under DownloadRepoThread
            def display_download_critical_error(self, errorText, friendlyErrorText=None, friendlySecondaryText=None):
                # TODO: make more friendly error displayer
                # TODO: support secondary text
##                handy.show_error(errorText, errorText)
                print >> sys.stderr, errorText
                if friendlyErrorText is None:
                    friendlyErrorText = errorText
                gobject.idle_add(lambda:self.parent.messagebox(friendlyErrorText, gtk.MESSAGE_ERROR, self.parent.win, friendlySecondaryText))
                gobject.idle_add(lambda:self.fetchWin.hide())
##                gobject.idle_add(lambda:self.parent.switch_to_toggled(self.parent.tgInstalled))
                    
                    
                
        # we're still under if not app.repoLoaded:
        fetchWin.show_all()

        app.currentDownloadRepoThread = DownloadRepoThread(app.repoURL, download_hook, app, fetchWin)
        app.currentDownloadRepoThread.start()

def clear_repo_cache(app):
    """deletes all cached information about the repository"""
    # TODO: clear_repo_cache, is this correct?
    if not DEBUG:
        subprocess.call(handy.join_lists(["rm", "-r"],glob(os.path.join(MY_CACHE_HOME, "*"))))
    client = gconf.client_get_default()
    app.repoLastModified = ""
    client.set_string("/apps/epidermis/repo_last_modified","")
    app.repoETag = ""
    client.set_string("/apps/epidermis/repo_etag","")
    pass


def main():
    print "This file should not be run"
    sys.exit(1)

if __name__ == "__main__":
    main()
