#!/usr/bin/env python
# Copyright 2008 David D Lowe
# released under the GPLv2

from pigment import PigmentType, Pigment
import pigment
import thread
import os, sys
from glob import glob
sys.path.append("..")
import handy
import ep_exceptions
import hashlib


class GDMType(PigmentType):
    humanName = {"en":"GDM"}
    humanPluralName = {"en":"GDM"}
    codeName = "gdm"
    abbreviation = "gdm"
    directoryName = "gdm"
        

class GDM(Pigment):
    
    
    def __init__(self):
        Pigment.__init__(self)
        self.type = GDMType()
        self.gdmDirname = ""
        self.gdmPath = ""
    
    def add_attachment(self, attachmentPath):
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if os.path.isabs(attachmentPath) and os.path.exists(attachmentPath):
            realPath = attachmentPath
        else:
            if os.path.exists(os.path.join("/usr/share/gdm/themes", attachmentPath.split("/")[-1])):
                realPath = os.path.join("/usr/share/gdm/themes", attachmentPath.split("/")[-1])
            else:
                self.gdmPath = ""
                self.gdmDirname = ""
                raise(ep_exceptions.AttachmentNotFoundException(attachmentPath))
        self.gdmPath = realPath
        self.gdmDirname = attachmentPath.split("/")[-1]
        Pigment.add_attachment(self, realPath)
        
    
    def install(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, query):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        #def ignore(progress): return False
        # TODO: check if overwrite, possibly ask progressCallback whether to overwrite
        if progressCallback is None: progressCallback = lambda progress, query: False
        if progressCallback(0.75, None): return
        #shell.do(["mkdir", "-p", "/usr/share/gdm/themes/%s" % self.gdmDirname])
        shell.do(["cp", "-r", self.gdmPath, "/usr/share/gdm/themes/"])
        progressCallback(1.0, None)
            
    
    def remove(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.remove(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        #def ignore(progress): return False
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        shell.do(["rm", "-r", "/usr/share/gdm/themes/%s" % self.gdmDirname])
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if shell is None:
            print >> sys.stderr, "shell is None, should include a shell with root acces for GDM"
            import shell
            shell = shell.SubprocessShell()
            shell.prepare()
        if self.gdmDirname == "" or self.gdmDirname is None:
            print >> sys.stderr, "GDM object (" + str(self) + ") has no gdmDirname"
            raise(Exception("GDM object has no gdmDirname"))
        if progressCallback(0.0): return
        from ConfigParser import SafeConfigParser # for reading INI files
        class BetterConfigParser(SafeConfigParser):
            """Better than SafeConfigParser: it's case-sensitive"""
            def __init__(self):
                SafeConfigParser.__init__(self)
            def optionxform(self, optionstr):
                return optionstr
        configParser = BetterConfigParser()
        configParser.read("/etc/gdm/gdm.conf-custom")
        configParser.set("greeter", "GraphicalTheme", self.gdmDirname)
        configParser.set("greeter", "GraphicalThemes", "happygnome:/" + self.gdmDirname) # I'm not sure why this line is necessary
        if not os.path.exists("/etc/gdm/gdm.conf-custom.epidermisinit.bak"):
            shell.do(["cp", "/etc/gdm/gdm.conf-custom", "/etc/gdm/gdm.conf-custom.epidermisinit.bak"])
        import subprocess
        subprocess.call(["mkdir", "-p", "/tmp/epidermis"])
        subprocess.call(["touch", "/tmp/epidermis/gdm.conf-custom"])
        configParser.write(open("/tmp/epidermis/gdm.conf-custom", "w"))
        shell.do(["mv", "-f", "/tmp/epidermis/gdm.conf-custom", "/etc/gdm/gdm.conf-custom"])
        # Can't seem to get this to work with the shell, fortunately, it doesn't need to be run
        # as root, so I'll use subprocess
        subprocess.call("gdmflexiserver --command=\"UPDATE_CONFIG greeter/GraphicalTheme\"", shell=True)
        subprocess.call("gdmflexiserver --command=\"UPDATE_CONFIG greeter/GraphicalThemes\"", shell=True)

        progressCallback(1.0)
    
    
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        xmlBit = doc.createElement("dirname")
        tn = doc.createTextNode(self.gdmDirname)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc

    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        attachmentPath = filePath[:(-1*len(filePath.split("/")[-1]))]
        self.add_attachment(os.path.join(attachmentPath, "data", pigment.get_first_text( \
            root.getElementsByTagName("dirname")[0])))
        self.gdmDirname = pigment.get_first_text(root.getElementsByTagName("dirname")[0])    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.isdir("/usr/share/gdm/themes/%s" % self.gdmDirname):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()
    
    def convert_from_meat(self, meat, progressCallback=None):
        """Automatically grabs information from 'meat'. Note: this won't install
        the pigment
        meat: the name of the file or directory of the installed customisation
        progressCallback: callback function, similar to the one passed to install"""
        # TODO: convert_from_meat
        pass
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = glob(self.gdmPath + "/*") 
            f2 = glob("/usr/share/gdm/themes/%s/*" % self.gdmDirname)
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = glob(self.gdmPath + "/*")
            from ConfigParser import SafeConfigParser # for reading INI files
            class BetterConfigParser(SafeConfigParser):
                """Better than SafeConfigParser: it's case-sensitive"""
                def __init__(self):
                    SafeConfigParser.__init__(self)
                def optionxform(self, optionstr):
                    return optionstr
            configParser = BetterConfigParser()
            configParser.read("/etc/gdm/gdm.conf-custom")
            nn = configParser.get("greeter", "GraphicalTheme")
            f2 = glob("/usr/share/gdm/themes/%s/*" % nn)
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return True
    
    def match_filter(self, string, lang="en"):
        """Returns whether this pigment should be a result for a search for the
        string passed"""
        for ii in (self.get_human_name(lang), self.get_description(lang)):
            if string.lower() in ii.lower():
                return True
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = GDM()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()