#!/usr/bin/env python
# Copyright 2008 David D Lowe
# released under the GPLv2

from pigment import PigmentType, Pigment
import pigment
import os, sys
import thread
import gconf
import ep_exceptions
import hashlib
import subprocess
from glob import glob
sys.path.append("..")
import handy


class MetacityType(PigmentType):
    humanName = {"en":"Metacity"}
    humanPluralName = {"en":"Metacity"}
    codeName = "metacity"
    abbreviation = "mt"
    directoryName = "metacity"
        

class Metacity(Pigment):
    
    
    def __init__(self):
        Pigment.__init__(self)
        self.type = MetacityType()
        self.metacityDirname = ""
        self.metacityPath = ""
    
    def add_attachment(self, attachmentPath):
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if os.path.isabs(attachmentPath) and os.path.exists(attachmentPath):
            realPath = attachmentPath
        else:
            if os.path.exists(os.path.join("/usr/share/themes", attachmentPath.split("/")[-1], "metacity-1")):
                realPath = os.path.join("/usr/share/themes", attachmentPath.split("/")[-1])
            else:
                self.metacityPath = ""
                self.metacityDirname = ""
                raise(ep_exceptions.AttachmentNotFoundException(attachmentPath))
        self.metacityPath = realPath
        self.metacityDirname = attachmentPath.split("/")[-1]
        Pigment.add_attachment(self, realPath)
    
    
    def install(self, installationDirectory=None, progressCallback=None,shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query=None: False
        def change_progress_callback(progress, query=None):
            if progressCallback(progress / 2, None):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        # copy metacity directory
        #def ignore(progress): return False
        # TODO: check if overwrite, possibly ask progressCallback whether to overwrite
        if progressCallback is None: progressCallback = lambda progress, query=None: False
        if progressCallback(0.75, None): return
        shell.do(["mkdir", "-p", "/usr/share/themes/%s/" % self.metacityDirname])
        shell.do(["cp", "-r", os.path.join(self.metacityPath, "metacity-1"),"/usr/share/themes/%s/" % self.metacityDirname])
        progressCallback(1.0, None)
            
    
    def remove(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.remove(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
            
        # remove metacity dir
        #def ignore(progress): return False
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        shell.do(["rm", "-r", "/usr/share/themes/%s/metacity-1" % self.metacityDirname])
        deleteWhole = True
        for ff in glob("/usr/share/themes/%s" % self.metacityDirname):
            if os.path.isdir(ff):
                deleteWhole = False
                break
        if deleteWhole:
            shell.do(["rm", "-r", "/usr/share/themes/%s" % self.metacityDirname])
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        client = gconf.client_get_default()
        client.set_string("/apps/metacity/general/theme", self.metacityDirname)
        progressCallback(1.0)

    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        attachmentPath = filePath[:(-1*len(filePath.split("/")[-1]))]
        self.add_attachment(os.path.join(attachmentPath, "data", pigment.get_first_text( \
            root.getElementsByTagName("dirname")[0])))
    
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        xmlBit = doc.createElement("dirname")
        tn = doc.createTextNode(self.metacityDirname)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.exists("/usr/share/themes/%s/metacity-1" % self.metacityDirname):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()
    
    def convert_from_meat(self, meat, progressCallback=None):
        # TODO: convert_from_meat
        pass
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = glob(self.metacityPath + "/metacity-1/*") # = file(self.metacityPath, 'rb')
            f2 = glob("/usr/share/themes/%s/metacity-1/*" % self.metacityDirname) #file(os.path.join("/usr/share/", self.codeName, self.wallpaperFilename), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        client = gconf.client_get_default()
        try:
            f1 = glob(self.metacityPath + "/metacity-1/*")
            f2 = glob("/usr/share/themes/%s/metacity-1/*" % client.get_string("/apps/metacity/general/theme")) 
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return False
    
    def match_filter(self, string, lang="en"):
        """Returns whether this pigment should be a result for a search for the
        string passed"""
        for ii in (self.get_human_name(lang), self.get_description(lang)):
            if string.lower() in ii.lower():
                return True
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = Metacity()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()